unit AgentPlugin;

interface

uses
  //WCu
  SysUtils,
  Classes,
  VCL.Forms,
  VCL.Menus,
  VCL.StdCtrls,
  VCL.Controls,
  VCL.ExtCtrls,
  VCL.Graphics,
  VCL.Dialogs,
  VCL.MPlayer,
  MMSYSTEM,
  ComObj,
  GL,
  Math,
  Windows,
  ScktComp,
  IdUDPClient,
  IdUDPBase,
  IdSocketHandle,

   //SDKCu
   PluginCore,
   F8Utils,
   F8PaintBox,
   F8GLUtils,
   F8GLText;

type

//ԗ
CarData = record
  x: Double; //ʒu(X)
  y: Double; //ʒu(Y)
  dx: Double; //(X)
  dy: Double; //(Y)
  sh: Double; //ikmj
  ss: Double; //bimj
  steering: Double; //XeAO
  throttle: Double; //ANZ
  brake: Double; //u[L
end;

//NX̒`
AgentPluginClass = class(TF8PLuginClass,IF8OpenGLPlugin)
    private

      //XVC^[o
      const INTERVAL = 500;

      //IPAhX
      const IP = '127.0.0.1';

      //|[gԍ
      const PORT = 54321;

      //XN[
      var SCREEN_WIDTH: Integer;

      //XN[
      var SCREEN_HEIGHT: Integer;

      //CEeLXg
      var MAIN_TEXT: String;

      //x
      var state_speed: Integer;

      //u[L
      var state_brake: Integer;

      //XeAO
      var state_steering: Integer;

      //NCAg\PbgiUDPj
      var udp_socket: TIdUDPClient;

      //ԗ
      var car_data: CarData;

      //X^[g
      var start_time: Longint;

      //oߎ
      var elapsed_time: Longint;

      //
      procedure init();

      //ԗ̑M
      procedure sendCarData();

      //ԗ̍XV
      procedure updateCarData();

      //XV
      procedure update();

      //ԗ̕`
      procedure paintCarData();

      //fBXvC`
      procedure PaintHUD();
      procedure PaintScene();
      procedure SetupCamera();

      //V~[V̊JnCxg
      procedure startSimEvent();

      //V[V̏ICxg
      procedure finishSimEvent();

    public

      //AvP[VƂ̒ʐM
      public winRoadApplication: IF8ApplicationServices;

      //ftHg錾
      procedure AfterConstruction; override;
      procedure BeforeDestruction; override;


    end;


//vOC̓o^
procedure RegisterUserPlugin(out optionName, apiVersion : String);

//vOC̃[h
procedure LoadPlugin;

//vOC̉
procedure UnloadPlugin;

var

    agent_plugin : AgentPluginClass;


implementation

//
procedure AgentPluginClass.init();
  begin

    //XN[ETCY
    SCREEN_WIDTH := winRoadApplication.mainForm.openGL.Width;
    SCREEN_HEIGHT := winRoadApplication.mainForm.openGL.Height;

    //l̐ݒ
    state_speed := 0;
    state_brake := 0;
    state_steering := 0;

    //Ԃ̐ݒ
    start_time := GetTickCount();
    elapsed_time := 0;


  end;

//ԗ̍XV
procedure AgentPluginClass.updateCarData();
  var
    driver: IF8MainDriver;
  begin
    driver := winRoadApplication.mainForm.driver;

     if assigned(driver) and assigned(driver.currentCar) then
      begin
        car_data.x := driver.currentCar.instancePosition[_x];
        car_data.y := driver.currentCar.instancePosition[_z];
        car_data.dx := driver.currentCar.direction[_x];
        car_data.dy := driver.currentCar.direction[_z];
        car_data.sh := driver.currentCar.speedInKmPerHour;
        car_data.ss := driver.currentCar.speedInMetresPerSecond;
        car_data.steering := driver.currentCar.steering;
        car_data.throttle := driver.currentCar.throttle;
        car_data.brake := driver.currentCar.brake;
      end;

  end;

//XV
procedure AgentPluginClass.update();
  begin

    //ԗ̍XV
    updateCarData();

    //ԗ̑M
    sendCarData();

  end;


//M
procedure AgentPluginClass.sendCarData();
  var
    info: String;
    response: String;
  begin

      elapsed_time := GetTickCount() - start_time;

      if elapsed_time >= INTERVAL then
        begin

          //\PbgJ
          udp_socket := TIdUDPClient.Create(nil);
          udp_socket.Host := IP;
          udp_socket.Port := PORT;
          udp_socket.Active := True;

          //bZ[W̍쐬
          info := Format('%4.2f', [car_data.x]) + ',' + Format('%4.2f', [car_data.y]) + ',' +
                  Format('%4.2f', [car_data.dx]) + ',' + Format('%4.2f', [car_data.dy]) + ',' +
                  Format('%4.2f', [car_data.steering]) + ',' + Format('%4.2f', [car_data.throttle]) + ',' +
                  Format('%4.2f', [car_data.brake]) + ',' + Format('%4.2f', [car_data.sh]) + ',AgentPlugin';

          //bZ[W̑M
          udp_socket.Send(info);

          //\Pbg
          udp_socket.Active := False;
          udp_socket := nil;

          //M̃Zbg
          start_time := GetTickCount();

        end;


  end;

//ԗ̕`
procedure AgentPluginClass.paintCarData();
  begin
    MAIN_TEXT :=  '[IP:' + IP + ' PORT:' + IntToStr(PORT) + '] ' +
                  '[X:' + Format('%4.1f', [car_data.x]) + ' Y:' + Format('%4.1f', [car_data.y]) + '] ' +
                  '[DX:' + Format('%4.2f', [car_data.dx]) + ' DY:' + Format('%4.2f', [car_data.dy]) + '] '+
                  '[SPEED(km):' + Format('%4.2f', [car_data.sh]) +
                  ' HANDLE:' + Format('%4.2f', [car_data.steering]) + ' ACCEL:' + Format('%4.2f', [car_data.throttle]) +
                  ' BRAKE:' + Format('%4.2f', [car_data.brake]) + ']';

    //tHgATCYAFݒ
    F8TextSetFontName('MSPSVbN');
    F8TextSetFontSize(20);
    F8TextSetFontColor(clWhite);

    //eLXgiԗj̕`ʒu
    F8TextSetAlignment(_alignCenter);
    F8TextStartFrom(0.5 * SCREEN_WIDTH, SCREEN_HEIGHT - 2.0 * 20);

    //eLXgiԗj̕`
    F8TextOut(MAIN_TEXT);

    //tHgATCYAFݒ
    F8TextSetFontName('MSPSVbN');
    F8TextSetFontSize(30);
    F8TextSetFontColor(clRed);

  end;

//fBXvC`
procedure AgentPluginClass.PaintHUD();
  begin
      //OpenGL̐ݒύX
      glDisable(GL_LIGHTING);
      glEnable(GL_BLEND);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

      //eLXg\Cu
      F8TextResetSettings();

      //2D[hJn
      F8Text2DModeBegin(SCREEN_WIDTH, SCREEN_HEIGHT);

      //ԗ̕`
      paintCarData();

      //XV
      update();

      //2D[hI
      F8Text2DModeEnd();

      //OpenGL̐ݒɖ߂
      glEnable(GL_LIGHTING);
      glDisable(GL_COLOR_MATERIAL);
  end;

//V[̕`igpj
procedure AgentPluginClass.PaintScene();
  begin
  end;

//J̐ݒigpj
procedure AgentPluginClass.SetupCamera();
  begin
  end;

//V~[V̊JnCxg
procedure AgentPluginClass.startSimEvent();
  begin
    init();
  end;

//V~[V̏ICxg
procedure AgentPluginClass.finishSimEvent();
  begin
  end;

//Cxg̓o^
procedure AgentPluginClass.AfterConstruction;
  var
    method: TMethod;
  begin
   Supports(ApplicationServices, IF8ApplicationServices, winRoadApplication);

   //V~[V̊JnCxg̓o^
   PluginNotifyProc(method) := startSimEvent;
   winRoadApplication.RegisterEventHandler(_plgSimulationStartDriving, method);

   //V~[V̒~Cxg̓o^
   PluginNotifyProc(method) := finishSimEvent;
   winRoadApplication.RegisterEventHandler(_plgSimulationStartDriving, method);

  end;

//Cxg̓o^
procedure AgentPluginClass.BeforeDestruction;
  var
    method: TMethod;
  begin

    //V~[VJnCxg̍폜
    PluginNotifyProc(method) := startSimEvent;
    winRoadApplication.UnRegisterEventHandler(_plgSimulationStartDriving, method);

     //V~[V~Cxg̓o^
    PluginNotifyProc(method) := finishSimEvent;
    winRoadApplication.UnRegisterEventHandler(_plgSimulationStopDriving, method);

    winRoadApplication := nil;
  end;

//vOC̓o^
procedure RegisterUserPlugin(out optionName, apiVersion : String);
  begin
  optionName := 'AgentPlugin';
  apiVersion := PLUGIN_VERSION;
  end;

//vOC̃[h
procedure LoadPlugin;
  begin
    if not Assigned(agent_plugin) then
      agent_plugin := AgentPluginClass.Create;

    if Assigned(agent_plugin) and Assigned(agent_plugin.winRoadApplication) then
      agent_plugin.winRoadApplication.RegisterPluginObject(agent_plugin);
  end;

//vOC̉
procedure UnloadPlugin;
  begin
    if Assigned(agent_plugin) and Assigned(agent_plugin.winRoadApplication) then
      agent_plugin.winRoadApplication.UnRegisterPluginObject(agent_plugin);

    agent_plugin.Free();
    agent_plugin := nil;
  end;

end.
