#coding: utf-8

import math
import time

class RobohonAction():

    IGNORE = -99

    def __init__(self, tokens):

        for i in range(len(tokens)):
            tokens[i] = str.strip(tokens[i])
            if(tokens[i] == "*"):
                tokens[i] = RobohonAction.IGNORE

        self.id = int(tokens[0])
        self.x = float(tokens[1])
        self.y = float(tokens[2])
        self.radius = float(tokens[3])
        self.dx = float(tokens[4])
        self.dy = float(tokens[5])
        self.diff = float(tokens[6])
        self.focus = int(tokens[7])
        self.handle = float(tokens[8])
        self.accel = float(tokens[9])
        self.brake = float(tokens[10])
        self.speed = float(tokens[11])
        self.action = int(tokens[12])
        self.comment = tokens[13]

        if len(tokens) == 15:
            self.sound = "sound/" + tokens[14]
        else:
            self.sound = ""

        self.active = True

    def toStringAll(self):
        text = "[ID=" + str(self.id) + " ACT=" + str(self.active) + " X=" + str(self.x) + " Y=" + str(self.y) + " RAD=" + str(self.radius) + " DX=" + str(self.dx) + " DY=" + str(self.dy) + " DIF=" + str(self.diff) + " FOC=" + str(self.focus) + " HDL=" + str(self.handle) + " ACL=" + str(self.accel) + " BRK=" + str(self.brake) + " SPD=" + str(self.speed) + " ACT=" + str(self.action)  + " COM=" + str(self.comment) + "]"
        return text

    def toString(self):
        text = "[ID=" + str(self.id) +  " COM=" + str(self.comment) + " SOUND=" + self.sound + "]"
        return text

    def toList(self, simdata):
        list = []

        list.append(time.time())
        list.append(simdata.x)
        list.append(simdata.y)
        list.append(simdata.dx)
        list.append(simdata.dy)
        list.append(simdata.handle)
        list.append(simdata.accel)
        list.append(simdata.brake)
        list.append(simdata.speed)
        list.append(self.id)
        list.append(self.focus)
        list.append(self.action)
        list.append(self.comment)
        list.append(self.sound)
        return list

    def check(self, simdata):
        position_flg = True
        angle_flg = True
        focus_flg = True
        handle_flg = True
        accel_flg = True
        brake_flg = True
        speed_flg = True

        # 位置条件
        if self.radius != RobohonAction.IGNORE:
            distance = math.sqrt((simdata.x - self.x) ** 2 + (simdata.y - self.y) ** 2)
            position_flg = (distance < self.radius)

        # 角度条件
        if self.diff != RobohonAction.IGNORE:
            angle_flg = ((self.dx - self.diff) <= simdata.dx) and (simdata.dx <= (self.dx + self.diff)) and ((self.dy - self.diff) <= simdata.dy) and (simdata.dy <= (self.dy + self.diff))

        # 焦点条件
        if self.focus != RobohonAction.IGNORE:
            focus_flg = (self.focus == simdata.focus)

        # ハンドル角条件
        if self.focus != RobohonAction.IGNORE:
            handle_flg = (self.handle <= abs(simdata.handle))

        # アクセル条件
        if self.accel != RobohonAction.IGNORE:
            accel_flg = (self.accel <= simdata.accel)

        # ブレーキ条件
        if self.brake != RobohonAction.IGNORE:
            brake_flg = (self.brake <= simdata.brake)

        # スピード条件
        if self.speed != RobohonAction.IGNORE:
            speed_flg = (self.speed <= simdata.speed)

        return (position_flg and angle_flg and focus_flg and handle_flg and accel_flg and brake_flg and speed_flg)

    def setActive(self, flg):
        self.active = flg
