#coding: utf-8

import socket
import csv
import Tkinter as tk
import pygame.mixer
from datetime import datetime
from SimData import SimData
from RobohonAction import RobohonAction

# ロボホンの設定ファイル
ROBOHON_FILE = "robohon/robohon-nozomi-normal.csv"
#ROBOHON_FILE = "robohon/robohon-nozomi-emo-a.csv"
#ROBOHON_FILE = "robohon/robohon-nozomi-emo-j.csv"
#ROBOHON_FILE = "robohon/robohon-nozomi-emo-s.csv"

# ロボホンとの通信
COMM_ROBOHON = False

# サウンドファイルの再生
PLAY_SOUND = True

# サウンドのリピート再生
PLAY_REPEAT = False

# ログ・ディレクトリ
LOG_DIR = "./log/"

class RobohonClient():

    # コンストラクタ
    def __init__(self, addr, port, message_length):
        self.addr = addr
        self.port = port
        self.message_length = message_length
        self.action = None

        self.actions = []
        self.robohon_file = open(ROBOHON_FILE, "r");
        self.reader = csv.reader(self.robohon_file)
        header = next(self.reader);
        for line in self.reader:
            robohon_action = RobohonAction(line)
            self.actions.append(robohon_action)

        self.log_file = open((datetime.now().strftime(LOG_DIR + "log-%Y-%m-%d-%H-%M-%S") + ".csv"), "w")
        self.writer = csv.writer(self.log_file, lineterminator="\n")

        list = []
        list.append("time")
        list.append("x")
        list.append("y")
        list.append("dx")
        list.append("dy")
        list.append("handle")
        list.append("accel")
        list.append("brake")
        list.append("speed")
        list.append("id")
        list.append("focus")
        list.append("action")
        list.append("comment")
        list.append("sound")
        self.writer.writerow(list)

        pygame.mixer.init()

    # デストラクタ
    def __del__(self):
        self.robohon_file.close()
        self.log_file.close()

    # データ送信
    def send(self, simdata):

        # アクションの選択
        for action in self.actions:
            if(action.check(simdata)):
                if(action.active == True):
                    #if(self.action != None):
                    #    self.action.setActive(True)
                    self.action = action
                    break

        # アクションの表示・記録
        if(self.action != None):
            if(self.action.active == True):
                self.writer.writerow(self.action.toList(simdata))
                print(self.action.toString())


        # サウンドの再生
        if PLAY_SOUND:
            if(self.action != None):
                if(self.action.active == True):
                    pygame.mixer.music.load(self.action.sound)
                    pygame.mixer.music.play()

                    # サウンドのリピート
                    if PLAY_REPEAT == False:
                        self.action.setActive(False)

        # ロボホンとの通信
        if COMM_ROBOHON:
            # IPV4(AF_INET), TCP/IP(SOCK_STREAM)でソケット作成
            client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

            # サーバーに接続
            client.connect((self.addr, self.port))

            # サーバーにアクションを送信
            if(self.action != None):
                if(self.action.active == True):
                    client.send(str(self.action.action) + "\r\n")
                    self.action.setActive(False)

                    # メッセージの取得
                    message = client.recv(self.message_length)
                    message = message.replace("\n", "")
                    print(message)

            # サーバーの接続終了
            client.close()
