# coding: UTF-8

import socket
import csv
import threading
import sys
import select
from datetime import datetime
from RobohonClient import RobohonClient
from SimData import SimData

# ロボホンのアドレス・ポート
ROBOHON_ADDR = "192.168.12.16"
ROBOHON_PORT = 10023

# ロボホンのメッセージサイズ
MESSAGE_LENGTH = 1024

# 最大接続数
MAX_CONNECTION = 10

# ログ・ディレクトリ
LOG_DIR = "./log/"

class SocketThread(threading.Thread):

    def __init__(self, addr, port, timeout, length):
        threading.Thread.__init__(self)
        self.loop_flg = False
        self.addr = addr
        self.port = port
        self.timeout = timeout
        self.length = length
        self.focus = -1
        self.focus_x = -1
        self.focus_y = -1
        self.robohon = RobohonClient(ROBOHON_ADDR, ROBOHON_PORT, MESSAGE_LENGTH)

        self.log_file = open((datetime.now().strftime(LOG_DIR + "log-eye-%Y-%m-%d-%H-%M-%S") + ".csv"), "w")
        self.writer = csv.writer(self.log_file, lineterminator="\n")

        list = []
        list.append("time")
        list.append("x")
        list.append("y")
        list.append("dx")
        list.append("dy")
        list.append("handle")
        list.append("accel")
        list.append("brake")
        list.append("speed")
        list.append("focus")
        list.append("focus_x")
        list.append("focus_y")
        list.append("comment")
        self.writer.writerow(list)

    # デストラクタ
    def __del__(self):
        self.log_file.close()

    def run(self):

        # IPV4(AF_INET), TCP/IP(SOCK_STREAM)でソケット作成
        server = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        # サーバーのIPアドレスとポートを指定
        server.bind((self.addr, self.port))

        # フラグの設定
        self.loop_flg = True

        # サーバー接続の開始
        print("[Start Socket]")
        while self.loop_flg:
            try:

                rready, wready, xready = select.select([server], [], [], self.timeout)

                for sock in rready:
                    if sock is server:

                        # クライアントからメッセージ取得
                        message = server.recv(self.length)
                        #print("[" + datetime.now().ctime() + " " + message + "]")

                        # メッセージの処理
                        if(message == "quit" or message == "QUIT"):
                            self.loop_flg = False
                        else:
                            # データを整形
                            simdata = SimData()
                            simdata.set(message, self.focus, self.focus_x, self.focus_y)
                            print(simdata.toString())

                            # ロボホンにメッセージを送信
                            self.robohon.send(simdata)

                            # ログに記録
                            self.writer.writerow(simdata.toCSV())
            except KeyboardInterrupt:
                self.loop_flg = False
            except socket.timeout:
                # タイムアウト処理
                print("[Session Timeout]")

        # サーバーを閉じる
        print("[Stop Socket]")
        server.close()

    # フォーカスの設定
    def setFocus(self, focus, focus_x, focus_y):
        self.focus = focus
        self.focus_x = focus_x
        self.focus_y = focus_y

    # スレッドの停止
    def stop(self):
        self.loop_flg = False
